/**************************************************************************
 *
 * lists.h -- List processing functions for boolean queries
 * Copyright (C) 1994  Neil Sharman, Alistair Moffat and Lachlan Andrew
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: lists.h 655 1999-10-11 02:58:42Z cs025 $
 *
 **************************************************************************/


#ifndef H_LISTS
#define H_LISTS

typedef struct query_data;
extern double FetchDocStart(query_data *, u_long, u_long *, u_long *);

class abstractDocEntry
{
 public:
  int   DocNum;
  float Weight;

  // public:
  inline int docNum()
    {
      return DocNum;
    }

  inline float docWeight()
    {
      return Weight;
    }
};

class loadableDocEntry : public abstractDocEntry
{
 public:
  u_long SeekPos;	/* position in the text file in bytes */
  u_long Len;		/* length of the document in bytes */

  inline double FetchStart(query_data *qd)
    {
      return FetchDocStart(qd, DocNum, &SeekPos, &Len);
    }
};

class DocEntry : public loadableDocEntry
  {
  public:
    char *CompTextBuffer;
    DocEntry *Next;
    short or_included;		/*[TS:Mar/95] whether included in an AND of ORed-terms */

    inline void init(int doc, float weight)
      { 
	DocNum         = doc;
	Weight         = weight;
	SeekPos        = 0;
	Len            = 0;
	CompTextBuffer = NULL;
	Next           = NULL;
	or_included    = 0;
      }

    inline void copy(DocEntry *src)
      { *this = *src;
      }
  };

typedef struct DocList
  {
    int num;
    int total_retrieved; /* valid if > 0 */
    int is_approx;       /* whether total_retrieved is approximate */
    int list_size;
    DocEntry *DE;

    inline int size()
    { return num;
    }

    inline DocEntry *member(int at)
    { return &DE[at];
    }

    inline DocEntry *lastMember()
    { return member(num - 1);
    }

    inline void addMember(int doc, float weight)
    { member(num)->init(doc, weight);
      num ++;
    }

    inline void addMember(DocEntry *entry)
    { DE[num] = *entry;
      num ++;
    }

    inline void setMember(int at, DocEntry *entry)
    { DE[at] = *entry;
    }
  }
DocList;

DocList *DocList_create (int num);
DocList *DocList_resize (DocList * d, int num);
void     DocList_destroy (DocList *d);
DocList *DocList_intersect (DocList * List1, DocList * List2);
DocList *DocList_diff (DocList * List1, DocList * List2);
DocList *DocList_merge (DocList * List1, DocList * List2);
DocList *DocList_addDoc(DocList *List, int res);
void     DocList_reduceOrIncluded(DocList *List);

#ifdef __cplusplus

inline int DocList_isEmpty(DocList *List)
{ return (List == NULL || List->num == 0);
}

inline void DocList_setsize(DocList *List, int n)
{ List->num = n;
}

inline DocEntry *DocList_nextMember(DocList *List, DocEntry *entry)
{ entry ++;
  return entry; 
}

inline int DocList_memberValid(DocList *List, DocEntry *entry)
{ return (entry < (List->DE + List->num));
}


inline void DocList_moveMember(DocList *List, int to, int from)
{ List->DE[to] = List->DE[from];
}

#endif
#endif
