/**********************************************************************
 *
 * hlistbrowserclass.cpp -- 
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * A component of the Greenstone digital library software
 * from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: hlistbrowserclass.cpp 668 1999-10-14 22:59:35Z sjboddie $
 *
 *********************************************************************/

/*
   $Log$
   Revision 1.2  1999/10/14 22:59:34  sjboddie
   finished off browser classes

   Revision 1.1  1999/10/10 08:14:08  sjboddie
   - metadata now returns mp rather than array
   - redesigned browsing support (although it's not finished so
   won't currently work ;-)

 */


#include "hlistbrowserclass.h"
#include <assert.h>
#include "OIDtools.h"

hlistbrowserclass::hlistbrowserclass () {
}

hlistbrowserclass::~hlistbrowserclass () {
}

// returns the name that specifies the browserclass type
text_t hlistbrowserclass::get_browser_name () {
  return "HList";
}

text_t hlistbrowserclass::get_default_formatstring () {
  return "[link][highlight][Title][/highlight][/link]";
}

// need to change OID to it's child 
void hlistbrowserclass::processOID (cgiargsclass &args, recptproto *collectproto,
				    ostream &logout) {

  text_t OID = args["d"];
  bool is_classify = false;
  if (OID.empty()) {
    is_classify = true;
    OID = args["cl"];
  }

  FilterResponse_t response;
  text_tset metadata;
  metadata.insert ("childtype");
  text_t classifytype = "Invisible";
  text_t child_doctype;

  text_tarray OIDs;

  while (classifytype == "Invisible") {
    OIDs.erase (OIDs.begin(), OIDs.end());
    
    OIDs.push_back (OID);
    OIDs.push_back (OID + ".fc");

    get_info (OIDs, args["c"], metadata, false, collectproto, response, logout);

    OID = response.docInfo[1].OID;
    classifytype = response.docInfo[0].metadata["childtype"].values[0];
  }
  if (is_classify) args["cl"] = OID;
  else args["d"] = OID;
}

int hlistbrowserclass::output_section_group (ResultDocInfo_t &section, cgiargsclass &args,
					     int colnumber, format_t *formatlistptr, 
					     bool use_table, text_tset &metadata, bool &getParents,
					     recptproto *collectproto, displayclass &disp, 
					     outconvertclass &outconvert, ostream &textout, ostream &logout) {

  // get all siblings
  FilterResponse_t response;
  text_t &arg_d = args["d"];
  text_t &arg_cl = args["cl"];
  get_children (section.OID + ".pr", args["c"], metadata, getParents, 
		collectproto, response, logout);

					
  if (use_table || colnumber > 0) {
    textout << "<table><tr><td>";
    // get tab size
    text_t tab; int itab;
    disp.expandstring ("Global", "_tabwidth_", tab);
    itab = tab.getint();
    if (colnumber > 0) textout << outconvert << disp 
			       << "<img src=\"_httpimg_/space.gif\" width=" 
			       << (itab*colnumber) << ">";
    textout << "</td>\n";
  }

  textout << "<table><tr>\n";

  ResultDocInfo_tarray::iterator tsibling = response.docInfo.begin();
  ResultDocInfo_tarray::iterator esibling = response.docInfo.end();

  text_t icon, link;
  while (tsibling != esibling) {

    bool highlight = false;
    textout << "<td>";

    if ((*tsibling).OID == section.OID) {
      link.clear();
      highlight = true;
    } else {
      link = "<a href=\"_httpdocument_";
      if (!arg_d.empty()) 
	link += "&cl=" + arg_cl + "&d=" + (*tsibling).OID + "\">";
      else 
	link += "&cl=" + (*tsibling).OID + "\">";
    }

    textout << outconvert << disp << get_formatted_string (*tsibling, formatlistptr, link, icon, highlight);

    textout << "</td>\n";
    tsibling ++;
  }

  textout << "</tr></table>\n";
  if (use_table  || colnumber > 0) textout << "</tr></table>\n";
  return 0;
}

